function [beta nll] = learn_poisson_regression (y, X, LEARNING_MODE)
%
%   learning Poisson regression using 'glmfit' or 'lassoglm'
%
%   (Reference) http://www.mathworks.com/help//stats/lasso-regularization-of-generalized-linear-models.html
%
% -------
% INPUTS:
%
%   y :             responses
%   X :             predictors
%   LEARNING_MODE : learning mode (See configuration file).
%
% --------
% OUTPUTS:
%
%   beta :          coefficient estimates
%   nll :           log-likelihood
%
% -------------
%
% Gunhee Kim


% K-fold cross validation
NUM_CV = 0 ;
% number of lambdas
NUM_LAMBDA = 10 ;
% a small number
EPS = 10^-4;

nll = [] ;

% (1) glmfit
if strcmp(LEARNING_MODE, 'glmfit'),
    %[beta, dev, stats] = glmfit(X, y, 'poisson') ;
    beta = glmfit(X, y, 'poisson') ;

% (2) lassoglm
elseif strcmp(LEARNING_MODE, 'lassoglm'),
    % cross validation ('without' is much faster than 'with').
    if NUM_CV==0,
        [B FitInfo] = lassoglm(X, y, 'poisson', 'NumLambda', NUM_LAMBDA);
        [~, minInd] = min(FitInfo.Deviance) ;
    else,
        [B FitInfo] = lassoglm(X, y,'poisson', 'NumLambda', 'CV', NUM_CV);
        % two different ways. (see http://www.mathworks.com/help/toolbox/stats/lassoglm.html)
        % minInd = FitInfo.IndexMinDeviance ;
        minInd = FitInfo.Index1SE ;
    end
    beta =[FitInfo.Intercept(:,minInd); B(:,minInd)] ;

    % compute unbiased coefficients
    % ignore the covariates of 0, and re-fit without any penalty terms.
    lenB = length(beta) ;
    bDelInd = find(beta(2:end)<EPS & beta(2:end)>-EPS) ;
    bAvilInd = 1:lenB-1 ; bAvilInd(bDelInd) = [] ;

    [unbiasedB,dev,stats] = glmfit(X(:,bAvilInd), y, 'poisson') ;

    beta = zeros(lenB,1) ;
    beta([1 bAvilInd+1]) = unbiasedB ;
else
    ;
end

% compute log-likelihood
% Refer http://www.mathworks.com/support/solutions/en/data/1-4K47XJ/index.html
if nargout>1,
    mu = glmval(beta, X, 'log') ;
    logval = log(poisspdf(y, mu)) ;
    logval(logval<EPS) = EPS ;
    nll = -sum(logval) ;
end

end
